!> \file temp_col.f90
!! Subroutines for vertical temperature calculation 
!<

!> \namespace temp_col
!!  Subroutines for vertical temperature calculation 
!<

!> SUBROUTINE: Temp_column 
!! Subroutine for vertical temperature calculation
!! Used modules:
!! - use Icetemp_declar
!! - use Tridiagmod
!>

Subroutine  Temp_col(Nz,Nzm,Nn,Newtempcol,Ctij,Flotij,Hij,Tbmerij,Ibase,Tij,Tpmpij,Aa,Bb,Cc,Rr,Hh,Ncond,Dee,Cm, &
				Dzm,Phidij,Ghfij,Ifail1,Tbdotij,DTT,Dou,Tsij)

!  Use Icetemp_declar
  Use Tridiagmod, only: tridiag
  
  
  Implicit None
  integer,intent(in) :: Nz,Nzm,Nn
  Real,Dimension(Nz+Nzm),Intent(Out)  :: Newtempcol        !< Tableau 1d Vert. Des Temperatures En Sortie
  Real,Dimension(Nz),Intent(In)       :: Ctij                !< Tableau 1d Vert. Conductivite Thermique
  Logical,Intent(In)                  :: Flotij              !< Vrai Si Flotijtant (Test D'Archimede) 'O'
  Real,Intent(In)                     :: Hij                 !< Ice Thickness  'O'
  Real,Intent(In)                     :: Tbmerij             !< Temperature De La Mer A La Base De L'Ice Shelf
  integer,intent(inout)               :: Ibase
  real,dimension(Nz+Nzm),intent(inout) :: Tij             !< Temperature sur la colonne
  real,dimension(Nz),intent(in)       :: Tpmpij               !< pressure melting point temperature in ice sheet 'o'
  Real,Dimension(Nn),intent(inout) :: Aa    !< Work Arrays For Tridiag  !Dim Nn
  Real,Dimension(Nn),intent(inout) :: Bb    !< Work Arrays For Tridiag  !Dim Nn
  Real,Dimension(Nn),intent(inout) :: Cc    !< Work Arrays For Tridiag  !Dim Nn
  Real,Dimension(Nn),intent(inout) :: Rr    !< Work Arrays For Tridiag  !Dim Nn
  Real,Dimension(Nn),intent(inout) :: Hh    !< Work Arrays For Tridiag  !Dim Nn
  Integer,intent(in)               :: Ncond !< Switch : Ncond=1 Thermal Conductivity In Mantle
  real, intent(in) :: Dee
  real,intent(in) :: Cm
  real,intent(in) :: Dzm
  real,intent(inout) :: Phidij     !< flux de chaleur lie a la deformation et glissement basal
  real,intent(in) :: Ghfij      !< flux geothermique
  integer,intent(out) :: Ifail1 !< Diagnostique erreur dans Tridiag
  real, intent(out) :: Tbdotij  !< variation in time of basal temperature
  real, intent(in) :: DTT
  Real, intent(inout) :: Dou
  real, intent(in) :: Tsij        !< surface ice temperature  'o'
  
  
  
  ! variables locales
  integer :: K
  Real :: Dzi,Tss
  Real,Dimension(Nzm+1) :: Abis,Bbis,Cbis,Rbis,Hbis   

  If (Hij.Gt.10.) Then

     ! Conditions Aux Limites a La Base De La Glace
     !________________________________________________

     ! Base Froide 
     If ( ((Ibase.Eq.1).Or.(Ibase.Eq.4) &
          .Or. ((Ibase.Eq.5).And.(Tij(Nz).Lt.Tpmpij(Nz)))) &
          .And..Not.Flotij) Then

        Ibase=1
        Bb(Nz)=1.

        If (Ncond.Eq.1) Then ! Avec Socle
           Dzi=Hij*Dee*Cm/Ctij(Nz)
           Aa(Nz)=-Dzm/(Dzm+Dzi)
           Cc(Nz)=-Dzi/(Dzm+Dzi)
           Rr(Nz)=Hij*Dee*Phidij*Dzm/Ctij(Nz)/(Dzm+Dzi)
        Else                 ! Sans Socle
           Aa(Nz)=-1.
           Cc(Nz)=0.
           Rr(Nz)=-(Ghfij-Phidij)/Ctij(Nz)*Hij*Dee
        Endif

     Else

        ! Base Temperee Ou Shelf
        ! ----------------------               
        If (Ibase.Eq.5)  Ibase=2
        Ibase=Max(Ibase,2)
        Aa(Nz)=0.
        Bb(Nz)=1.
        Cc(Nz)=0.

        If (.Not.Flotij) Then
           Rr(Nz)=Tpmpij(Nz)
        Else
           Rr(Nz)=Tbmerij
        End If

     Endif


     If (Ncond.Eq.1) Then ! Avec Socle
        Do K=Nz+1,nz+nzm-1
           Rr(K)=Tij(K)
        End Do
        Call Tridiag (Aa,Bb,Cc,Rr,Hh,nz+nzm,Ifail1)

     Else                 ! Sans Socle

        Call Tridiag (Aa,Bb,Cc,Rr,Hh,Nz,Ifail1)
     Endif


     ! Temperature Dans Le Socle Si Ncond=0, Lineaire Avec Le Gradient Ghf
     If (Ncond.Eq.0) Then
        Do K=Nz+1,nz+nzm
           Hh(K)=Hh(Nz)-Dzm*(K-Nz)*Ghfij/Cm
        End Do
     Endif

     Do K=1,nz+nzm
        Newtempcol(K)=Hh(K)
     End Do
     Tbdotij=(Newtempcol(Nz)-Tij(Nz))/Dtt

     !      ------------------------------------------------- Glace Tres Fine (H<10m Ou H=0)
  Else If (Hij.Le.10.) Then

     !       Pour Eviter Des Problemes Lorsque H Est Inferieur A 10 M
     !       Profil Lineaire, Pente=Flux Geothermique
     !       Le Cas Sans Glace Est Traite De La Meme Facon  (Dou=0)

     ! ........................................ Avec Calcul Dans Le Socle
     If (Ncond.Eq.1.And..Not.Flotij) Then   
        If (Hij.Gt.0.) Then
           ! Gradient Dans Le Socle
           Dou=(Tij(Nz+1)-Tij(Nz))/Dzm*Cm     
           Dou=Dou/Ctij(Nz)*Dee*Hij

           Tss=Min(0.,Tsij)
           Do K=1,Nz
              Newtempcol(K)=Tss+Dou*(K-1.)
           End Do

        Else
           ! Pas De Glace
           Tss=Tsij
           Do K=1,Nz
              Newtempcol(K)=Tss
           End Do
        End If

        ! Calcul Dans Le Socle Meme S'Il N'Y A Pas De Glace
        Rr(Nz)=Newtempcol(Nz)
        Aa(Nz)=0
        Bb(Nz)=1
        Cc(Nz)=0

        Do K=Nz+1,nz+nzm-1
           Rr(K)=Tij(K)
        End Do

        ! Creation De Nouveaux Tableaux Juste Pour Tridiag      
        Do K=1,Nzm+1
           Abis(K)=Aa(Nz-1+K)
           Bbis(K)=Bb(Nz-1+K)
           Cbis(K)=Cc(Nz-1+K)
           Rbis(K)=Rr(Nz-1+K)
        End Do

        Call Tridiag (Abis,Bbis,Cbis,Rbis,Hbis,Nzm+1,Ifail1)

        Do K=1,Nzm+1
           Hh(Nz-1+K)=Hbis(K)
        End Do

        ! ........................................ Sans Calcul Dans Le Socle
     Else
        ! Calotte Posee                    
        If ((Hij.Gt.0.).And..Not.Flotij) Then
           Dou=-Ghfij/Ctij(Nz)*Dee*Hij
           Tss=Min(0.,Tsij)
           Do K=1,Nz
              Newtempcol(K)=Tss+Dou*(K-1.)
           End Do

           !  Shelf
        Else If ((Hij.Gt.0.).And.Flotij) Then
           Tss=Min(0.,Tsij)
           Dou=(Tbmerij-Tss)*Dee
           Do K=1,Nz
              Newtempcol(K)=Tss+Dou*(K-1.)
           End Do

           ! Pas De Glace
        Else
           Tss=Tsij      
           Do K=1,Nz
              Newtempcol(K)=Tss
           End Do
        End If

        ! Temperature Dans Le Socle, Lineaire Avec Le Gradient Ghf
        If (.Not.Flotij) Then
           Do K=Nz+1,nz+nzm
              Hh(K)=Newtempcol(Nz)-Dzm*(K-Nz)*Ghfij/Cm
           End Do
        Else
           Do K=Nz+1,nz+nzm
              Hh(K)=Tbmerij-Dzm*(K-Nz)*Ghfij/Cm
           End Do
        End If

     Endif
     ! Fin Du Test Socle


     Do K=Nz+1,nz+nzm
        Newtempcol(K)=Hh(K)
     End Do

     Tbdotij=(Newtempcol(Nz)-Tij(Nz))/Dtt
     Ibase=5
     Phidij=0.


  End If

End Subroutine Temp_col
